<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2003-2011 Renzo Lauper (renzo@churchtool.org)
*  All rights reserved
*
*  This script is part of the kOOL project. The kOOL project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license
*  from the author is found in LICENSE.txt distributed with these scripts.
*
*  kOOL is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
 

/**
 * Class to render list views
 * This is the preferred way to create lists in kOOL. It is based on
 * the smarty template ko_list2.tpl.
 * It takes the definitions from $KOTA to render the list.
 */
class kOOL_listview {
	var $_editColumns = array("chk", "chk2", "edit", "delete", "check", "forward", "undelete");
	var $tmpl = array();  //Holds the values to be submitted to smarty on rendering
	var $doFooter = FALSE;
	var $disableMultiedit = FALSE;
	var $disableAccessRights = FALSE;


  /**
   * Initialise the list with some base informations
   * @param module: The kOOL module in which this list is used
   * @param table: Database table as defined in $KOTA
   * @param editColumns: Which action columns (edit, delete, etc) should be shown
   * @param start: Set the first element to be shown in a paged view
   * @param limit: Set the limit of items to be shown on one page
   */
	function init($module, $table, $editColumns="", $start="", $limit="") {
		$this->module = $module;
		$this->table = $table;
		if($editColumns != "") $this->setEditColumns($editColumns);
		if($start != "") $this->start = $start;
		if($limit != "") $this->limit = $limit;
		$this->session_id = session_id();
	}//init()


	function disableMultiedit() {
		$this->disableMultiedit = TRUE;
	}//disableMultiedit()

	function disableListCheckAll() {
		$this->disableListCheckAll = TRUE;
	}//disableListCheckAll()

	function disableKotaProcess() {
		$this->disableKotaProcess = TRUE;
	}//disableKotaProcess()


	function setTitle($title) {
		$this->tmpl["list_title"] = $title;
	}//setTitle

	function setSubTitle($title) {
		$this->tmpl["list_subtitle"] = $title;
	}//setSubTitle

	
	/**
	  * Set what columns should be displayed to the left (chk, edit, delete etc.)
	  */
	function setEditColumns($cols) {
		if(!is_array($cols)) {
			$cols = explode(",", $cols);
		}
		foreach($cols as $col) {
			if(!in_array($col, $this->_editColumns)) continue;
			$this->editColumns[$col] = $col;
		}
	}//setEditColumns()


	/**
	  * Set Statistic information for paging
		*/
	function setStats($total, $start="", $limit="") {
		if($start != "") $this->start = $start;
		if($limit != "") $this->limit = $limit;

		//Paging stats
		$this->tmpl["stats"] = array("start" => $this->start, "end" => min( ($this->start+$this->limit-1), $total), "total" => $total, "oftotal" => getLL("list_oftotal"));

		//Links for prev and next page
		if($this->start > 1) {
			$this->tmpl["paging"]["prev"] = "javascript:sendReq('../".$this->module."/inc/ajax.php', 'action,set_start,sesid', 'setstart,".max($this->start-$this->limit, 1).",".session_id()."', do_element);";
		} else {
			$this->tmpl["paging"]["prev"] = "";
		}
		if(($this->start+$this->limit-1) < $total) {
			$this->tmpl["paging"]["next"] = "javascript:sendReq('../".$this->module."/inc/ajax.php', 'action,set_start,sesid', 'setstart,".($this->limit+$this->start).",".session_id()."', do_element);";
		} else {
			$this->tmpl["paging"]["next"] = "";
		}
	}//setStats()


	/**
	  * Define a list footer
		*/
	function setFooter($footer) {
		if($footer) {
			$this->doFooter = TRUE;
			$this->tmpl["list"]["footer"] = array("show" => TRUE, "data" => $footer);
		}
	}//setFooter


	/**
	  * Render the list
		* Second argument may be xls to get an Excel file
		* Third parameter may be the file prefix for XLS export
		*/
	function render($data, $mode="html", $file_prefix="") {
		global $KOTA, $smarty, $ko_path;

		//Sort listview for the key
		ksort($KOTA[$this->table]["_listview"]);

		//Find columns to be displayed in the list view
		$show_cols = array();
		foreach($KOTA[$this->table]["_listview"] as $c) {
			if($c["name"]) $show_cols[] = $c["name"];
		}

		//Multiedit
		$multiedit = array();
		if(!$this->disableMultiedit && ($this->disableAccessRights || $this->access['MAX'] >= $this->accessLevels['edit'])) {
			foreach($KOTA[$this->table]["_listview"] as $col) {
				if(isset($col["multiedit"]) && $col["multiedit"] == FALSE) {
					$multiedit[] = "";
				} else {
					$multiedit[] = $col["multiedit"] != "" ? $col["multiedit"] : $col["name"];
				}
			}
			$this->tmpl["show_multiedit"] = TRUE;
			$this->tmpl["multiedit_cols"] = $multiedit;
		} else {
			$this->tmpl["show_multiedit"] = FALSE;
		}

		//Prepare header
		foreach($KOTA[$this->table]["_listview"] as $col) {
			$col["name"] = getLL("kota_listview_".$this->table."_".$col["name"]);
			$header[] = $col;
		}
		$this->tmpl["list"]["header"] = $header;

		//Sorting
		$this->tmpl["list"]["sort"] = $this->sort;


		$render_data = array(); $row_counter = 0;
		//Go through all rows to be displayed
		foreach($data as $id => $value) {
			//Process data
			if(!$this->disableKotaProcess) kota_process_data($this->table, $value, "list", $log, $id);
			//Assign values to list view array
			foreach($show_cols as $col) {
				if($this->columnLink[$col] && $mode == "html") {  //Add link for this column
					$render_data["data"][$row_counter][$col] = $this->getColumnLink($col, $data, $id, $value[$col]);
				} else {
					$render_data["data"][$row_counter][$col] = $value[$col];
				}
			}
			//Set access rights
			$render_data['meta'][$row_counter] = $this->getActions($data[$id]);  //Don't pass $value, as this has already been processed
			//Set special class for this row
			$render_data["meta"][$row_counter]["rowclass"] = $this->getRowClass($data, $id);
			//Set id
			$render_data["meta"][$row_counter]["id"] = $id;

			$row_counter++;
		}

		if(is_array($this->colParams)) $render_data["meta"]["colparams"] = $this->colParams;

		//Set values for template
		$this->tmpl["list"]["data"] = $render_data["data"];
		$this->tmpl["list"]["meta"] = $render_data["meta"];
		$this->tmpl["list"]["actions"] = $this->listActions;

		//Render as HTML (default)
		if($mode == "html") {
			$this->smarty_assign();
			$smarty->display("ko_list2.tpl");
		}
		//Create XLS file and store filename in this->xls_file
		else if($mode == "xls") {
			$xls_data = $this->tmpl["list"]["data"];
			foreach($this->tmpl["list"]["header"] as $row) {
				$xls_header[] = $row["name"];
			}
			$this->xls_file = $ko_path."download/excel/".$file_prefix.strftime("%d%m%Y_%H%M%S", time()).".xls";
			ko_export_to_excel($xls_header, $xls_data, $this->xls_file, "");
		}
	}//render()



	/**
	 * Get additional classes for single rows based on set conditions (see setRowClass())
	 */
	function getRowClass($data, $id) {
		$r = "";

		//Prepare map array from raw data
		foreach($data[$id] as $k => $v) {
			$map[strtoupper($k)] = $v;
		}

		foreach($this->rowClasses as $class => $cond) {
			$condition = strtr($cond, $map);
			if($condition != "" && eval($condition)) {
				$r .= " ".$class;
			}
		}
		return $r;
	}//getRowClass()



	/**
	 * Set additional classes for rows based on conditions
	 */
	function setRowClass($class, $cond) {
		$this->rowClasses[$class] = $cond;
	}//setRowClass()



	function setColParams($params) {
		$this->colParams = $params;
	}



	
	/**
	  * Sets a link for a certain column
		*/
	function setColumnLink($col, $link, $cond="") {
		$this->columnLink[$col] = $link;
		if($cond != "") {
			$this->columnLinkCondition[$col] = $cond;
		}
	}




	/**
	  * Generates a link for a value in a specified column
		* col: Column to be linked
		* data: Raw data
		* id: ID of the current line within the data array
		* value: the formated value to be linked (already preprocessed with kota_process_data(, , list, , )
		*/
	function getColumnLink($col, $data, $id, $value) {
		//Prepare map array from raw data
		foreach($data[$id] as $k => $v) {
			$map[strtoupper($k)] = $v;
		}

		//Generate link with formated value as text
		if(!isset($this->columnLinkCondition[$col]) ||
				($this->columnLinkCondition[$col] && eval(str_replace("@VALUE@", $value, $this->columnLinkCondition[$col])))
			) {
			$link = strtr($this->columnLink[$col], $map);
			return '<a href="'.$link.'">'.$value.'</a>';
		} else {
			return $value;
		}
	}//getColumnLink()



	/**
	  * Set edit, delete etc. buttons for a single row
		*/
	function getActions(&$data) {
		global $KOTA;

		$chk_col = $KOTA[$this->table]['_access']['chk_col'];
		$r = array();
		foreach($this->editColumns as $type) {
			if(substr($this->accessLevels[$type], 0, 3) == 'ALL') {
				if($this->disableAccessRights || $this->access['ALL'] >= str_replace('ALL', '', $this->accessLevels[$type])) $r[$type] = TRUE;
			} else {
				if($this->disableAccessRights || $this->access[$data[$chk_col]] >= $this->accessLevels[$type]) $r[$type] = TRUE;
			}
		}
		return $r;
	}//getActions()



	/**
	  * Supply arrays with access rights for the entries to be displayed
		* Or set FALSE to disable check for access rights
		*/
	function setAccessRights($level, &$access) {
		if($level == FALSE) {
			$this->disableAccessRights = TRUE;
		} else {
			//Check for all rights to be higher than all access levels
			$all = TRUE;
			foreach($level as $k => $v) {
				if($access['ALL'] < $v) $all = FALSE;
			}
			if($all) {  //Disable access rights if all rights are higher than all levels
				$this->disableAccessRights = TRUE;
			} else {
				$this->accessLevels = $level;
				$this->access = $access;
			}
		}
	}//setAccessRights()



	/**
	  * Supply actions to be performed for each editColumn
		*/
	function setActions($data) {
		foreach($this->editColumns as $type) {
			$this->listActions[$type] = $data[$type];
		}
	}//setActions()



	/**
	  * Define sorting
		*/
	function setSort($enable, $action="", $current="", $current_order="") {
		if($enable) {
			$this->sort = array("show" => $enable, "action" => $action, "akt" => $current, "akt_order" => $current_order);
		} else {
			$this->sort = array("show" => FALSE);
		}
	}//setSort()



	/**
	  * Assign all necessary values to smarty
		*/
	function smarty_assign() {
		global $smarty;

		foreach($this->tmpl as $key => $value) {
			$smarty->assign($key, $value);
		}
		$smarty->assign("edit_cols", $this->editColumns);

		//LL values
		$label = array("alt_edit" => getLL("list_label_edit_entry"),
									 "alt_check" => getLL("list_label_check_entry"),
									 "alt_send" => getLL("list_label_send_entry"),
									 "alt_undelete" => getLL("list_label_undelete_entry"),
									 "alt_delete" => getLL("list_label_delete_entry"),
									 "confirm_delete" => getLL("list_label_confirm_delete"),
									 );
		$smarty->assign("label", $label);

		if($this->disableListCheckAll) $smarty->assign("list_check_disabled", TRUE);

		//session id and module (for ajax calls)
		$smarty->assign("sesid", session_id());
		$smarty->assign("module", $this->module);

		//Help link
		$smarty->assign("help", ko_get_help($this->module, $_SESSION["show"]));
	}//smarty_assign()
	
}//kOOL_listview
?>
