<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2003-2011 Renzo Lauper (renzo@churchtool.org)
*  All rights reserved
*
*  This script is part of the kOOL project. The kOOL project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*  A copy is found in the textfile GPL.txt and important notices to the license
*  from the author is found in LICENSE.txt distributed with these scripts.
*
*  kOOL is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
//TODO: - Filter presets as recipients (filter.)
//      - SMS instead of email (sms.)

error_reporting(E_ALL);

//Only allow call from cli
if(!isset($argc) || $argc < 1) exit;

//Get ko_path from server settings
if($_SERVER['PHP_SELF']) {
	$PATH_thisScript = str_replace('//','/', str_replace('\\','/', $_SERVER['PHP_SELF']));
} else {
	$PATH_thisScript = str_replace('//','/', str_replace('\\','/', $_ENV['_']));
}
$ko_path = dirname($PATH_thisScript).'/';

if(isset($_POST['GLOBALS']) || isset($_GET['GLOBALS'])) die('You cannot set the GLOBALS-array from outside this script.');


$ko_menu_akt = 'mailing';
require($ko_path.'inc/ko.inc');

//Basic checks
if(defined('ALLOW_SEND_EMAIL') && ALLOW_SEND_EMAIL == FALSE) exit;
if(!in_array('mailing', $MODULES)) exit;
if(!is_array($MAILING_PARAMETER) || sizeof($MAILING_PARAMETER) < 3) exit;


//Get mailing parameters from ko-config and ko_settings
$host   = $MAILING_PARAMETER['host'];
$port   = $MAILING_PARAMETER['port'];
$user   = $MAILING_PARAMETER['user'];
$pass   = $MAILING_PARAMETER['pass'];
$domain = $MAILING_PARAMETER['domain'];
$ssl    = $MAILING_PARAMETER['ssl'];
$folder = $MAILING_PARAMETER['folder'];

//Number of email to be sent by cycle
$mails_per_cycle = ko_get_setting('mailing_mails_per_cycle');
if($mails_per_cycle < 1) $mails_per_cycle = 30;
if($mails_per_cycle > 50) $mails_per_cycle = 50;
//Maximum number of recipients
$max_recipients = ko_get_setting('mailing_max_recipients');
if($max_recipients == '') $max_recipients = 0;

//Set Return-Path for sent emails
if(check_email($MAILING_PARAMETER['return_path'])) {
	$return_path = '-f'.$MAILING_PARAMETER['return_path'];
} else if(defined('EMAIL_SET_RETURN_PATH') && EMAIL_SET_RETURN_PATH == TRUE) {
	$return_path = '-f'.ko_get_setting('info_email');
} else {
	$return_path = '';
}


// Constants: Mailing status
define('MAILING_STATUS_OPEN', 1);
define('MAILING_STATUS_CONFIRMED', 2);
define('MAILING_STATUS_SENT', 3);

//Contants: Errors
define('ERROR_INVALID_GROUP_ID', 1);
define('ERROR_INVALID_SMALLGROUP_ID', 2);
define('ERROR_INVALID_SENDER', 3);
define('ERROR_INVALID_CODE', 4);
define('ERROR_NO_ALIAS_FOUND', 5);
define('ERROR_INVALID_RECIPIENT', 6);
define('ERROR_NON_UNIQUE_ALIAS', 7);
define('ERROR_TOO_MANY_RECIPIENTS', 8);
define('ERROR_NO_ACCESS', 9);
define('ERROR_GROUP_NO_ACCESS', 10);
define('ERROR_SMALLGROUP_NO_ACCESS', 11);
define('ERROR_MYLIST_NO_ACCESS', 12);



//Create POP3 connection
$ssl = ($ssl==false)?'/novalidate-cert':'';
$folder = $folder ? $folder : 'INBOX';
$imap = imap_open('{'."$host:$port/pop3$ssl"."}$folder",$user,$pass);

//Exit if connection to IMAP failed
if($imap == FALSE) {
	db_insert_data('ko_log', array('type' => 'mailing_error', 'comment' => getLL('mailing_error_imap'), 'date' => date('Y-m-d H:i:s')));
	exit;
}

$done_mailids = array();
$done_error_mails = array();


//Get emails from pop account
if($num_mails = imap_num_msg($imap) > 0) {
	//Get mails
	$mails = array();
	$response = imap_fetch_overview($imap,'1:'.$num_mails);
	foreach ($response as $msg) $mails[$msg->msgno] = (array)$msg;

	foreach($mails as $mail) {
		//Get header info
		$header = imap_rfc822_parse_headers(imap_fetchheader($imap, $mail['msgno']));

		//Check message id
		//If one mails has two or more recipients, the same mail will be stored multiple times, so we ignore all but the first one
		//For each mail all recipients will be handled below, so no need to work through all copies
		if(in_array($mail['message_id'], $done_mailids)) {
			imap_delete($imap, $mail['msgno']);
			continue;
		}

		//Get all recipients of this email
		$mail_recipients = array();
		foreach($header->to as $obj) {
			if($obj->host != $domain) continue;  //ignore other hosts
			$mail_recipients[] = $obj->mailbox.'@'.$obj->host;
		}
		if($header->cc) {
			foreach($header->cc as $obj) {
				if($obj->host != $domain) continue;  //ignore other hosts
				$mail_recipients[] = $obj->mailbox.'@'.$obj->host;
			}
		}

		//Check sender email and find corresponding kOOL login
		$login_id = ko_mailing_get_sender_login($mail['from']);
		if(!$login_id) {
			ko_mailing_error(ERROR_INVALID_SENDER, $mail);
			//Delete message
			imap_delete($imap, $mail['msgno']);
			//Store message id
			$done_mailids[] = $mail['message_id'];
			continue;
		}
		ko_get_login($login_id, $db_user);

		//Check access rights for found user
		$access = array();
		//Access to the mailing module
		if(!ko_module_installed('mailing', $login_id)) {
			ko_mailing_error(ERROR_NO_ACCESS, $mail);
			continue;
		}
		//Access to people module
		ko_get_access('leute', $login_id);
		if(!ko_module_installed('leute', $login_id) || $access['leute']['MAX'] < 1) {
			ko_mailing_error(ERROR_NO_ACCESS, $mail);
			continue;
		}
		//Access rights for groups and smallgroups, check will be done further down
		ko_get_access('groups', $login_id);
		ko_get_access('kg', $login_id);


		$found = FALSE;
		foreach($mail_recipients as $mail_recipient) {
			$error = 0;
			$to = str_replace('@'.$domain, '', $mail_recipient);

			//Find mails sent to noreply (e.g. autoresponders)
			if($to == 'noreply') {
				$found = TRUE;
			}
			//Find confirm emails
			else if(substr($to, 0, strlen('confirm-')) == 'confirm-') {
				$code = substr($to, strlen('confirm-'));
				$error = ko_mailing_check_code($code);

				if($error) {
					ko_mailing_error($error, $mail, $to);
				} else {
					ko_mailing_mail_confirmed($code);
					$found = TRUE;
				}

			}
			//Find group with id
			else if(1 == preg_match('/^gr([0-9.]*$)/', $to, $m)) {
				list($all, $data) = $m;

				list($gid, $rid) = explode('.', $data);
				$error = ko_mailing_check_group($gid, $rid);

				if($error) {
					ko_mailing_error($error, $mail, $to);
				} else {
					$mail['_recipient'] = 'gr'.$gid.($rid ? '.'.$rid : '');
					ko_mailing_store_moderation($imap, $mail);
					$found = TRUE;
				}
			}
			//Find smallgroup with id
			else if(1 == preg_match('/^sg([0-9]{4})([a-zA-Z.]*)$/', $to, $m)) {
				list($all, $sgid, $rid) = $m;

				$error = ko_mailing_check_smallgroup($sgid, $rid);

				if($error) {
					ko_mailing_error($error, $mail, $to);
				} else {
					$mail['_recipient'] = 'sg'.$sgid.($rid?'.'.$rid:'');
					ko_mailing_store_moderation($imap, $mail);
					$found = TRUE;
				}
			}
			//My List
			else if($to == 'ml') {
				$error = ko_mailing_check_mylist();

				if($error) {
					ko_mailing_error($error, $mail, $to);
				} else {
					$mail['_recipient'] = 'ml';
					ko_mailing_store_moderation($imap, $mail);
					$found = TRUE;
				}
			}
			//Find mailing alias
			else {
				//Find group or small group with this alias
				$groups = db_select_data('ko_groups', "WHERE LOWER(`mailing_alias`) = '".mysql_real_escape_string(strtolower($to))."'");
				$smallgroups = db_select_data('ko_kleingruppen', "WHERE LOWER(`mailing_alias`) = '".mysql_real_escape_string(strtolower($to))."'");
				$num_found = sizeof($groups)+sizeof($smallgroups);
				if($num_found == 0) {
					ko_mailing_error(ERROR_NO_ALIAS_FOUND, $mail, $to);
				} else if($num_found > 1) {
					ko_mailing_error(ERROR_NON_UNIQUE_ALIAS, $mail, $to);
				} else {
					if(sizeof($groups) == 1) {
						$group = array_shift($groups);
						$error = ko_mailing_check_group($group['id']);
						if($error) {
							ko_mailing_error($error, $mail, $to);
						} else {
							$mail['_recipient'] = 'gr'.$group['id'];
							ko_mailing_store_moderation($imap, $mail);
							$found = TRUE;
						}
					} else if(sizeof($smallgroups) == 1) {
						$sg = array_shift($smallgroups);
						$error = ko_mailing_check_smallgroup($sg['id']);
						if($error) {
							ko_mailing_error($error, $mail, $to);
						} else {
							$mail['_recipient'] = 'sg'.$sg['id'];
							ko_mailing_store_moderation($imap, $mail);
							$found = TRUE;
						}
					}
				}
			}
		}//foreach(mail_recipients)

		//None of the email addresses with $domain have been recognized and processed: Return failure notice
		if(!$found) {
			ko_mailing_error(ERROR_INVALID_RECIPIENT, $mail);
		}

		//Delete message after it has been processed
		imap_delete($imap, $mail['msgno']);

		//Store message id, so it won't be processed again (two recipients generate 2 emails each with both recipients)
		$done_mailids[] = $mail['message_id'];

	}//foreach(mails as mail)
}

//Close connection and expunge
imap_close($imap, CL_EXPUNGE);





//Check db for mails to be sent
$mails = db_select_data('ko_mailing_mails', "WHERE `status` = '".MAILING_STATUS_CONFIRMED."'");
foreach($mails as $mail) {
	$done = $done_names = array();

	//Get next recipients and send emails
	$recipients = db_select_data('ko_mailing_recipients', "WHERE `mail_id` = '".$mail['id']."'", '*', '', 'LIMIT 0,'.$mails_per_cycle);
	foreach($recipients as $rec) {
		if(!$rec['id']) continue;
		$to = $rec['name'].' <'.$rec['email'].'>';
		$sent = mail($to, $mail['subject'], '', $mail['header'].$mail['body'], $return_path);
		$done[] = $rec['id'];
		$done_names[] = $to;
	}

	//Delete done recipients
	if(sizeof($done) > 0) db_delete_data('ko_mailing_recipients', "WHERE `id` IN ('".implode("','", $done)."')");

	//Create log entry with all recipients
	db_insert_data('ko_log', array('type' => 'mailing_sent', 'comment' => $mail['id'].': '.implode(', ', $done_names), 'user_id' => $mail['user_id'], 'date' => date('Y-m-d H:i:s')));

	//Check recipients, mark mail as sent if none left
	$num = db_get_count('ko_mailing_recipients', 'id', "AND `mail_id` = '".$mail['id']."'");
	if($num == 0) {
		db_update_data('ko_mailing_mails', "WHERE `id` = '".$mail['id']."'", array('status' => MAILING_STATUS_SENT));
		//Add log entry after finishing mailing
		$log = $mail['id'].': '.'Subject: '.$mail['subject'].', From: '.$mail['from'].', To: '.$mail['recipient'];
		db_insert_data('ko_log', array('type' => 'mailing_done', 'comment' => $log, 'user_id' => $mail['user_id'], 'date' => date('Y-m-d H:i:s')));
	}
}




//Check for old non-confirmed mails and delete them
$limit = add2date(date('Y-m-d'), 'day', '-5', TRUE).' 00:00:00';
$old_mails = db_select_data('ko_mailing_mails', "WHERE `status` = '".MAILING_STATUS_OPEN."' AND `crdate` < '".$limit."'");
foreach($old_mails as $mail) {
	$log = '';
	$logcols = array('id', 'crdate', 'recipient', 'from', 'subject');
	foreach($logcols as $c) $log .= (getLL('mailing_header_'.$c) ? getLL('mailing_header_'.$c) : $c).': '.$mail[$c].', ';
	db_insert_data('ko_log', array('type' => 'mailing_delete_old', 'comment' => substr($log, 0, -2), 'user_id' => $mail['user_id'], 'date' => date('Y-m-d H:i:s')));
	db_delete_data('ko_mailing_mails', "WHERE `id` = '".$mail['id']."'");
}







function ko_mailing_get_sender_login(&$from) {
	global $LEUTE_EMAIL_FIELDS;

	//Find sender email address
	$pos1 = strrpos($from, '<');
	if(FALSE !== $pos1) {
		$pos2 = strpos($from, '>', $pos1);
		$from = substr($from, $pos1 + 1, ($pos2 ? $pos2 : strlen($from)) - $pos1 - 1);
	}
	if(!check_email($from)) return FALSE;
	$from = strtolower($from);


	//email fields
	$where_email = '';
	foreach($LEUTE_EMAIL_FIELDS as $field) {
		$where_email .= " LOWER(l.$field) = '".$from."' OR ";
	}

	$logins = db_select_data("ko_admin AS a LEFT JOIN ko_leute as l ON a.leute_id = l.id",
												 "WHERE ($where_email LOWER(a.email) = '".mysql_real_escape_string($from)."') AND (a.disabled = '0' OR a.disabled = '')",
												 "a.id AS id");
	$login = array_shift($logins);
	return $login['id'];
}//ko_mailing_get_sender_login()




/**
 * Mark mail as confirmed and create recipient entries for all recipients in queue
 * @param $code code of th email that was confirmed
 */
function ko_mailing_mail_confirmed($code) {
	global $db_user;

	$mail = db_select_data('ko_mailing_mails', "WHERE `code` = '$code' AND `status` = '".MAILING_STATUS_OPEN."'", '*', '', '', TRUE);

	//Get recipients
	$recipients = ko_mailing_get_recipients($mail['recipient']);

	//Create db entries
	foreach($recipients as $r) {
		ko_get_leute_email($r, $emails);
		foreach($emails as $email) {
			$email = trim($email);
			if(!check_email($email)) continue;
			$entry = array('mail_id' => $mail['id'], 'name' => $r['vorname'].' '.$r['nachname'], 'email' => $email);
			db_insert_data('ko_mailing_recipients', $entry);
		}
	}

	//Set status of mail to confirmed
	db_update_data('ko_mailing_mails', "WHERE `id` = '".$mail['id']."'", array('status' => MAILING_STATUS_CONFIRMED));

	//Create log entry
	$log = $mail['id'].': '.'Subject: '.$mail['subject'].', From: '.$mail['from'].', To: '.$mail['recipient'].', Recipients: '.sizeof($recipients);
	db_insert_data('ko_log', array('type' => 'mailing_confirmed', 'comment' => $log, 'user_id' => $db_user['id'], 'date' => date('Y-m-d H:i:s')));
}//ko_mailing_mail_confirmed()




/**
 * Return a list of entries from ko_leute who are recipients of the given mailinglist
 * @param $rec sgXXXX[.Y], grXXXXXX.YYYYYY or ml
 */
function ko_mailing_get_recipients($rec) {
	global $access, $login_id;

	$mode = substr($rec, 0, 2);
	$data = substr($rec, 2);
	$_recipients = array();

	switch($mode) {
		case 'gr':
			$parts = explode('.', $data);
			$gid = array_shift($parts);
			$rid = array_shift($parts);
			$group = 'g'.$gid.($rid ? ':r'.$rid : '');
			$where = "WHERE `groups` REGEXP '$group' AND `deleted` = '0' AND `hidden` = '0'";
			$_recipients = db_select_data('ko_leute', $where);
		break;

		case 'ml':
			$ids = unserialize(ko_get_userpref($login_id, 'leute_my_list'));
			if(sizeof($ids) > 0) {
				$_recipients = db_select_data('ko_leute', "WHERE `id` IN ('".implode("','", $ids)."') AND `deleted` = '0' AND `hidden` = '0'");
			}
		break;

		case 'sg':
			$parts = explode('.', $data);
			$sgid = array_shift($parts);
			$rid = array_shift($parts);
			$_recipients = db_select_data('ko_leute', "WHERE `smallgroups` REGEXP '".$sgid.($rid?':'.$rid:'')."' AND `deleted` = '0' AND `hidden` = '0'");
		break;
	}

	//Perform tests
	$recipients = array();
	$done_emails = array();
	foreach($_recipients as $r) {
		//Check for access to this single address
		if($access['leute']['ALL'] < 1 && $access['leute'][$r['id']] < 1) continue;
		//Check for valid email
		if(FALSE === ko_get_leute_email($r, $emails)) continue;
		$email = trim(array_shift($emails));
		//Don't send mail to same address twice
		if(in_array($email, $done_emails)) continue;

		$recipients[] = $r;
		$done_emails[] = $email;
	}

	return $recipients;
}//ko_mailing_get_recipients()



function ko_mailing_get_rec_name($rec) {
	global $all_groups, $db_user;

	$mode = substr($rec, 0, 2);
	$data = substr($rec, 2);

	switch($mode) {
		case 'gr':
			if(!$all_groups) $all_groups = db_select_data('ko_groups', 'WHERE 1');

			$parts = explode('.', $data);
			$gid = array_shift($parts);
			//Get full id for the group
			$motherline = ko_groups_get_motherline($gid, $all_groups);
			$mids = array();
			foreach($motherline as $mg) {
				$mids[] = 'g'.$all_groups[$mg]['id'];
			}
			$full_id = (sizeof($mids) > 0 ? implode(':', $mids).':' : '').'g'.$gid;
			//Add role
			$rid = array_shift($parts);
			if($rid) $full_id .= ':r'.$rid;

			return getLL('mailing_group').' '.ko_groups_decode($full_id, 'group_desc_full');
		break;

		case 'sg':
			$parts = explode('.', $data);
			$sgid = array_shift($parts);
			$sg = db_select_data('ko_kleingruppen', "WHERE `id` = '".$sgid."'", '*', '', '', TRUE);
			return getLL('mailing_smallgroup').' '.$sg['name'];
		break;

		case 'ml':
			return $db_user['login']."'s ".getLL('mailing_mylist');
		break;
	}
}//ko_mailing_get_rec_name()




/**
 * Send error email to sender if valid and create log entry about the error
 * @param $error: Error number that occured
 * @param $mail: Mail array that caused the error
 * @param $to: If given, contains the current recipient that caused the problem
 */
function ko_mailing_error($error, $mail, $to='') {
	global $domain, $db_user, $done_error_mails, $max_recipients, $return_path;

	//Only send one error message for each mail
	if(in_array($mail['id'], $done_error_mails)) return;
	$done_error_mails[] = $mail['id'];

	if(check_email($mail['from'])) {
		//Prepare mapping array
		$map = array();
		$map['@MAX_RECIPIENTS@'] = $max_recipients;
		if($to) $map['@CURRENT_TO@'] = $to;
		foreach($mail as $k => $v) {
			$map['@'.strtoupper($k).'@'] = $v;
		}

		//Send error email
		$headers = 'From: noreply'.'@'.$domain;
		$message  = getLL('mailing_errormail_text_intro');
		$message .= strtr(getLL('mailing_errormail_text_'.$error), $map);
		$message .= "\n\n".ko_mailing_summary($mail);
		mail($mail['from'], getLL('mailing_errormail_subject'), $message, $headers, $return_path);
	}//if(check_email(from))


	//Create log entry
	$log = 'ERROR '.$error.' '.getLL('mailing_error_'.$error).' - ';
	foreach($mail as $k => $v) {
		if($k && $v) $log .= $k.': '.$v.', ';
	}
	if(substr($log, 0, -2) == ', ') $log = substr($log, 0, -2);
	db_insert_data('ko_log', array('type' => 'mailing_error', 'comment' => $log, 'user_id' => $db_user['id'], 'date' => date('Y-m-d H:i:s')));
}//ko_mailing_error()




function ko_mailing_store_moderation($imap, $mail) {
	global $db_user, $domain, $return_path;

	//Get user email
	$person = ko_get_logged_in_person($db_user['id']);

	//Get email body and create email code
	$body = imap_body($imap, $mail['msgno']);
	$code = md5(uniqid(mt_rand(), true));

	//Get header and delete some entries
	$header = imap_fetchheader($imap, $mail['msgno']);
	$header = preg_replace('/(\n|^)To: .*\n/i', '$1', $header);
	$header = preg_replace('/(\n|^)Cc: .*\n/i', '$1', $header);
	$header = preg_replace('/(\n|^)Bcc: .*\n/i', '$1', $header);
	$header = preg_replace('/(\n|^)Subject: .*\n/i', '$1', $header);
	$header = preg_replace('/(\n|^)Delivered-To: .*\n/i', '$1', $header);

	//Send confirm email to user
	$from = 'confirm-'.$code.'@'.$domain;
	$headers = 'From: '.$from."\n".'Reply-To: '.$from;
	$message = sprintf(getLL('mailing_confirm_text'), $from)."\n\n".ko_mailing_summary($mail, $body);
	mail($person['email'], getLL('mailing_confirm_subject').': '.$mail['subject'], $message, $headers, $return_path);

	//Create db entry for email
	$entry = array();
	$entry['body'] = $body;
	$entry['header'] = $header;
	$entry['status'] = MAILING_STATUS_OPEN;
	$entry['crdate'] = date('Y-m-d H:i:s');
	$entry['code'] = $code;
	$entry['subject'] = $mail['subject'];
	$entry['from'] = $mail['from'];
	$entry['user_id'] = $db_user['id'];
	$entry['recipient'] = $mail['_recipient'];
	$new_id = db_insert_data('ko_mailing_mails', $entry);

	//Create log entry
	$log = $new_id.': Subject: '.$mail['subject'].', From: '.$mail['from'].', To: '.$mail['to'].', Size: '.$mail['size'];
	db_insert_data('ko_log', array('type' => 'mailing_new', 'comment' => $log, 'user_id' => $db_user['id'], 'date' => date('Y-m-d H:i:s')));
}//store_mail_for_moderation()




function ko_mailing_summary($mail, $body='') {
	$summary = '';


	//Get name of recipient (group or other) for a nicer display
	$rec_name = ko_mailing_get_rec_name($mail['_recipient']);
	if($rec_name) $summary .= getLL('mailing_rec_name').': '.$rec_name."\n";

	//Add number of recipients
	$recipients = ko_mailing_get_recipients($mail['_recipient']);
	$summary .= getLL('mailing_number_of_recipients').': '.sizeof($recipients)."\n";

	//Show parts of the header
	$show_headers = array('from', 'to', 'subject', 'date');
	foreach($mail as $k => $v) {
		if(!in_array($k, $show_headers)) continue;
		$summary .= getLL('mailing_header_'.$k).': '.$v."\n";
	}

	//Add part of the body
	if($body) {
		$summary .= "\n".substr($body, 0, 200)."\n[...]\n";
	}

	return $summary;
}//ko_mailing_summary()



function ko_mailing_check_group($gid, $rid) {
	global $max_recipients, $access;

	//Check for correct gid
	if(!$gid || strlen($gid) != 6) return ERROR_INVALID_GROUP_ID;
	for($i=0; $i<strlen($gid); $i++) {
		if(!in_array(substr($gid, $i, 1), array(0,1,2,3,4,5,6,7,8,9))) return ERROR_INVALID_GROUP_ID;
	}

	//Check for access
	if($access['groups']['ALL'] < 1 && $access['groups'][$gid] < 1) return ERROR_GROUP_NO_ACCESS;

	//Check role id if given
	if($rid) {
		if(strlen($rid) != 6) return ERROR_INVALID_GROUP_ID;
		for($i=0; $i<strlen($rid); $i++) {
			if(!in_array(substr($rid, $i, 1), array(0,1,2,3,4,5,6,7,8,9))) return ERROR_INVALID_GROUP_ID;
		}
	}

	//Check for valid group in DB
	$group = db_select_data('ko_groups', "WHERE `id` = '$gid' AND (`start` < CURDATE() && (`stop` = '0000-00-00' OR `stop` >= CURDATE()))", 'id', '', '', TRUE);
	if(!$group['id'] || $group['id'] != $gid) return ERROR_INVALID_GROUP_ID;
	if($rid && !in_array($rid, explode(',', $group['roles']))) return ERROR_INVALID_GROUP_ID;

	//Check for recipients
	$recipients = ko_mailing_get_recipients('gr'.$gid.($rid ? '.'.$rid : ''));
	if(sizeof($recipients) <= 0) return ERROR_INVALID_GROUP_ID;
	if($max_recipients > 0 && sizeof($recipients) > $max_recipients) return ERROR_TOO_MANY_RECIPIENTS;

	return 0;
}//ko_mailing_check_group()



function ko_mailing_check_mylist() {
	global $max_recipients, $access;

	//Check for access
	if($access['leute']['MAX'] < 1) return ERROR_MYLIST_NO_ACCESS;

	//Check for recipients
	$recipients = ko_mailing_get_recipients('ml');
	if(sizeof($recipients) <= 0) return ERROR_MYLIST_NO_ACCESS;
	if($max_recipients > 0 && sizeof($recipients) > $max_recipients) return ERROR_TOO_MANY_RECIPIENTS;

	return 0;
}//ko_mailing_check_mylist()




function ko_mailing_check_smallgroup($sgid, $rid) {
	global $SMALLGROUPS_ROLES, $max_recipients, $access;

	//Check for correct sgid
	if(!$sgid || strlen($sgid) != 4) return ERROR_INVALID_SMALLGROUP_ID;
	for($i=0; $i<strlen($sgid); $i++) {
		if(!in_array(substr($sgid, $i, 1), array(0,1,2,3,4,5,6,7,8,9))) return ERROR_INVALID_SMALLGROUP_ID;
	}

	//Check for access
	if($access['kg']['ALL'] < 1) return ERROR_SMALLGROUP_NO_ACCESS;

	//Check role id if given
	if($rid) {
		if(strlen($rid) != 1) return ERROR_INVALID_SMALLGROUP_ID;
		if(!in_array($rid, $SMALLGROUPS_ROLES)) return ERROR_INVALID_SMALLGROUP_ID;
	}

	//Check for valid smallgroup in DB
	$sg = db_select_data('ko_kleingruppen', "WHERE `id` = '$sgid'", 'id', '', '', TRUE);
	if(!$sg['id'] || $sg['id'] != $sgid) return ERROR_INVALID_SMALLGROUP_ID;

	//Check for recipients
	$recipients = ko_mailing_get_recipients('sg'.$sgid.($rid?'.'.$rid:''));
	if(sizeof($recipients) <= 0) return ERROR_INVALID_SMALLGROUP_ID;
	if($max_recipients > 0 && sizeof($recipients) > $max_recipients) return ERROR_TOO_MANY_RECIPIENTS;

	return 0;
}//ko_mailing_check_smallgroup()





function ko_mailing_check_code($code) {
	//Check for valid md5 hash
	if(strlen($code) != 32) return ERROR_INVALID_CODE;
	for($i=0; $i<strlen($code); $i++) {
		if(!in_array(substr($code, $i, 1), array(0,1,2,3,4,5,6,7,8,9,'a','b','c','d','e','f'))) return ERROR_INVALID_CODE;
	}
	
	//Check db for mail with this code
	$mail = db_select_data('ko_mailing_mails', "WHERE `code` = '$code' AND `status` = '".MAILING_STATUS_OPEN."'", '*', '', '', TRUE);
	if(!$mail['id'] || $mail['code'] != $code) return ERROR_INVALID_CODE;

	return 0;
}//ko_mailing_check_code()

?>
